#This script should read in a file with information from emission lines (that have names) that come from Giacomo's multi gauss fitting scripts. It then recognises which lines are present, calculates and prints the line ratio's that are needed for the BPT diagrams and then makes the BPT diagrams.

import numpy as np
import matplotlib.pyplot as plt

#load the file with all the information
file = 'NGC3599_lines.txt'
lines = np.loadtxt(file, usecols=0, skiprows=1, dtype=str, unpack=True)
flux, dflux = np.loadtxt(file, usecols=(7,8), skiprows=1, unpack=True)
rel_error = dflux/flux

#make the data arrays so that flux["Halpha_narrow"] gives the flux of the narrow Halpha lines
flux = np.array(tuple(flux), dtype=[(name, float) for name in lines])
dflux = np.array(tuple(dflux), dtype=[(name, float) for name in lines])
rel_error = np.array(tuple(rel_error), dtype=[(name, float) for name in lines])

#calculate the line ratios for the combinations needed for the various BPT diagrams, if one of the lines is not present, give as output that this line is not present

#order to check if all the lines are present:
#Hbeta and OIII_5007 for the y-axis
#Halpha and NII_6585 for the NII BPT diagram x-axis
#SII both for the SII BPT diagram x-axis
#OI for the OI BPT diagram x-axis

#Keep track of which BPT-diagrams need to be made
Y, X1,  X2,  X3 = False, False, False, False #These become true is all lines are present for a certain BPT-diagram

print("Line ratios and errors:")

if np.all([line in lines for line in ["Hbeta_narrow", "OIII_5007"]]):
	Hb_O3_rat = flux["OIII_5007"]/flux["Hbeta_narrow"]
	Hb_O3_error = 0.434*np.sqrt(rel_error["OIII_5007"]**2+rel_error["Hbeta_narrow"]**2) #0.434 comes from 1/ln10 for the derivative 
	print("[OIII]/Hb$:", Hb_O3_rat, Hb_O3_error*Hb_O3_rat)
	Y=True
else:
	print("Spectrum does not contain on of the following lines: Hbeta_narrow or OIII_5007")

if np.all([line in lines for line in ["Halpha_narrow", "NII_6584"]]):
	Ha_N2_rat = flux["NII_6584"]/flux["Halpha_narrow"]
	Ha_N2_error = 0.434*np.sqrt(rel_error["NII_6584"]**2+rel_error["Halpha_narrow"]**2)
	print("[NII]/Ha$:", Ha_N2_rat, Ha_N2_error*Ha_N2_rat)
	X1=True
else:
	print("Spectrum does not contain on of the following lines: Halpha_narrow or OIII_6584")

if np.all([line in lines for line in ["Hbeta_narrow", "SII_6717", "SII_6731"]]):
	SII = flux["SII_6717"]+flux["SII_6731"]
	SII_rel_error = np.sqrt(dflux["SII_6717"]**2+dflux["SII_6731"]**2)/SII
	Ha_S2_rat = SII/flux["Halpha_narrow"]
	Ha_S2_error = 0.434*np.sqrt(SII_rel_error**2+rel_error["Halpha_narrow"]**2)
	print("[SII]/Ha$:", Ha_S2_rat, Ha_S2_error*Ha_S2_rat)
	X2=True
else:
	print("Spectrum does not contain on of the following lines: Halpha_narrow, SII_6717 or SII_6731")

if np.all([line in lines for line in ["Halpha_narrow", "OI_6300"]]):
	Ha_O1_rat = np.abs(flux["OI_6300"])/flux["Halpha_narrow"]
	Ha_O1_error = 0.434*np.sqrt(np.abs(rel_error["OI_6300"])**2+rel_error["Halpha_narrow"]**2)
	print("[OI]/Ha$", Ha_O1_rat, Ha_O1_error*Ha_O1_rat)
	X3=True
else:
	print("Spectrum does not contain on of the following lines: Halpha_narrow or OI_6300")

#make the demarkations for the devision of the BPT-diagrams in the different regions (HII, AGN, Seyfert, Composite)
xaxis = np.linspace
xaxis = np.linspace(-2.5, 1.0, 100)
#N2 diagram
Kaufmann03 = 0.61/(xaxis-0.05)+1.3
Kewley01 = 0.61/(xaxis-0.47)+1.19
#S2 diagram
AGN_S2 = 0.72/(xaxis - 0.32) + 1.30
Liner_S2 = 1.89*xaxis + 0.76
#O1 diagram
AGN_O1 = 0.73 / (xaxis + 0.59) + 1.33
Liner_O1 = 1.18*xaxis + 1.30

#masks to plot the correct part of all the lines
ymin = -1.2
ymax = 1.5
mask1 = xaxis <= 0.3
mask2 = xaxis <= 0.2
mask3 = xaxis <= -0.7
mask4 = (xaxis <= 0.3) & (Kaufmann03 <= Kewley01)
mask5 = (xaxis <= 0.4) & (Liner_S2 >= AGN_S2)
mask6 = (xaxis <= 0.5) & (Liner_O1 >= AGN_O1)

#make 1, 2 or 3 BPT-diagrams depending on which lines are present
if Y:
	fig, ax = plt.subplots(nrows=1, ncols=X1+X2+X3, sharey=True, gridspec_kw={'wspace':0})
	ax = np.atleast_1d(ax)
	i = 0
	#plt.ylabel(r"LOG ([OIII]/H$\beta$)")
	if X1:
		ax[i].plot(xaxis[mask4], Kaufmann03[mask4], 'k', linestyle='dashed', linewidth=.75)
		ax[i].plot(xaxis[mask1], Kewley01[mask1], 'k', linewidth=.75)
		ax[i].errorbar(np.log10(Ha_N2_rat), np.log10(Hb_O3_rat), Hb_O3_error, Ha_N2_error, 'rs', capsize=3)
		ax[i].set_ylim(ymin, ymax)
		ax[i].set_xlim(-2.0, 1.0)
		ax[i].text(-0.3, 1.2, 'AGN', fontsize='xx-large')
		ax[i].text(-1.3, -0.1, 'HII', fontsize='xx-large')
		ax[i].text(-0.3, -1.0, 'Comp', fontsize='xx-large')
		ax[i].set_ylabel(r"log ([OIII]$\lambda$5007/H$\beta$)", fontsize='x-large')
		ax[i].set_xlabel(r"log ([NII]$\lambda$6584/H$\alpha$)", fontsize='x-large')
		i += 1
	if X2:
		ax[i].plot(xaxis[mask2], AGN_S2[mask2], 'k', linewidth=.75)
		ax[i].plot(xaxis[mask5], Liner_S2[mask5], 'k', linewidth=.75)
		ax[i].errorbar(np.log10(Ha_S2_rat), np.log10(Hb_O3_rat), Hb_O3_error, Ha_S2_error, 'rs', capsize=3)
		ax[i].set_xlim(-1.2, 0.8)
		ax[i].text(-0.7, 1.2, 'Seyfert', fontsize='xx-large')
		ax[i].text(-1.0, -0.1, 'HII', fontsize='xx-large')
		ax[i].text(0.1, 0.0, 'LINER', fontsize='xx-large')
		if not X1:
			ax[i].set_ylabel(r"log ([OIII]$\lambda$5007/H$\beta$)", fontsize='x-large')
		ax[i].set_xlabel(r"log ([SII]$\lambda$6717, 6731/H$\alpha$)", fontsize='x-large')
		i += 1
	if X3:
		ax[i].plot(xaxis[mask3], AGN_O1[mask3], 'k', linewidth=.75)
		ax[i].plot(xaxis[mask6], Liner_O1[mask6], 'k', linewidth=.75)
		ax[i].errorbar(np.log10(Ha_O1_rat), np.log10(Hb_O3_rat), Hb_O3_error, Ha_O1_error, 'rs', capsize=3)
		ax[i].set_xlim(-2.2, 0.0)
		ax[i].text(-1.5, 1.2, 'Seyfert', fontsize='xx-large')
		ax[i].text(-1.7, -0.1, 'HII', fontsize='xx-large')
		ax[i].text(-0.5, -0.3, 'LINER', fontsize='xx-large')
		if not X1:
			ax[i].set_ylabel(r"LOG ([OIII]$\lambda$5007/H$\beta$)", fontsize='x-large')
		ax[i].set_xlabel(r"LOG ([OI]$\lambda$6300/H$\alpha$)", fontsize='x-large')
	#plt.title(file.split("_")[0])
	plt.savefig(file.split("_")[0]+"BPT.pdf", dpi=400)
	plt.show()
