'''
7 03 2019 Last version
Giacomo Cannizzaro
Multi-gaussian fit with lmfit package
https://lmfit.github.io/lmfit-py/index.html
'''

#*************************************#

#  REMEMBER TO UPDATE PREFIX LIST!!   #

#*************************************#

import numpy as np
import sys
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from lmfit import Model, Parameters

# Open file and normalise the flux
# file is opened from the command line
# ascii file with 3 columns: WL, flux, flux_error 
# >python multi_gauss_lmfit_general.py file.dat

infile = open(sys.argv[1],"r")
xspec,flux=np.loadtxt(infile,unpack=True)
 
error_file = open(sys.argv[2],"r")
wl_error, rel_error = np.loadtxt(error_file, unpack=True) 

mask = (wl_error <= xspec[-1])
eflux = np.abs(flux*rel_error[mask])
#xspec,flux,eflux=np.loadtxt(infile,unpack=True)

# correct for redshift to make it easier to know where to look for the lines
#xspec/=(1+0.002699) 

# flux normalisation - check your units
# select WL interval
flux*=1.e16
eflux*=1.e16
xmin=4830.
xmax=5030.
x=xspec[(xspec>xmin)&(xspec<xmax)]
y=flux[(xspec>xmin)&(xspec<xmax)]
ey=eflux[(xspec>xmin)&(xspec<xmax)]

#print(len(x), len(y), len(eflux))


# General definition of functions
# wl = x0 (centroid)
# x=x coordinate (wavelength for the spectrum)

def gauss(x,a,wl,sigma):
	return a*np.exp(-(x-wl)**2/(2*sigma**2))

def poly(x,slope,interc):
	return x*slope+interc



# Create a model with multiple gaussians and a slope
# to add a gaussian, you have to add Model(gauss, prefix='string')
# every gaussian will have a prefix (pref_) --> parameters are pref_a, pref_wl, pref_sigma
# calling 'make_params' will create parameters with the right prefix
# remember to update the list of prefixes, this is needed after for printing/plotting purposes

mod = Model(poly) + \
	  Model(gauss, prefix='line1_')+ \
	  Model(gauss, prefix='line2_')+ \
	  Model(gauss, prefix='line3_')

params = mod.make_params()

#*****************************************************************#

# this is important to update to get the right prints of results! #

#*****************************************************************#

prefixes = ['line1_','line2_','line3_']

# assigning initial values
# check https://lmfit.github.io/lmfit-py/constraints.html for example of possible constraints
# you can put min, max and set the parameter to vary or not, and tie them with mathematical expressions
# for every component you add to the model, you have to specify the components


params['slope'].set(value=-0.00006)
params['interc'].set(value=0.5)

#line1
params['line1_a'].set(value=21.)
params['line1_wl'].set(value=4861.)#, expr = 'line4_wl')
params['line1_sigma'].set(value=2.)#, expr='line3_sigma')

#line2
params['line2_a'].set(value=25., expr='line3_a/3.')
params['line2_wl'].set(value=4959., expr='line3_wl-47.9')
params['line2_sigma'].set(value=.5, expr = 'line3_sigma')

#line3
params['line3_a'].set(value=76.)
params['line3_wl'].set(value=5007.)
params['line3_sigma'].set(value=2.)


##Examples of expressions to fix values
## 1) fixing wav separation between two lines (1 and 2)
##    params['line2_wl'].set(value=4363.2,expr='line1_wl-643.7')
## 2) correlating FWHM of two lines (1 and 2)
##    params['line2_sigma'].set(value=5., expr='line1_sigma/line1_wl*line2_wl')



## mod.fit does the fit
## prints a report of the fit, chisq ecc
result = mod.fit(y, params, x=x, weights=1/ey)

print(result.fit_report(show_correl=False))
#print(result.fit_report())




#*********************#

#  PRINTING RESULTS   #

#*********************#

c=299792.458 #c in km/s
print('#line\t WV \t \t dWV \t FWHM(km/s) \t dFWHM(km/s) \t EW \t dEW \t flux \t \t dflux')


## values and errors for every parameter
for pref in prefixes:
	slop=result.params['slope'].value
	dslop=result.params['slope'].stderr
	dinter=result.params['interc'].stderr
	WL=result.params[pref+'wl'].value
	dWL=result.params[pref+'wl'].stderr
	sig=abs(result.params[pref+'sigma'].value) 
	dsig=result.params[pref+'sigma'].stderr 
	cont=result.params['slope'].value*WL+result.params['interc'].value
	FWHM=2.3548*sig
	dFWHM=2.3548*dsig
	FWkms=FWHM*c/WL
	dFWkms=c*2.3548*np.sqrt( (dsig/WL)**2 + (sig*dWL/WL**2)**2 ) 
	amp=result.params[pref+'a'].value
	damp=result.params[pref+'a'].stderr
	flux=2.50663*amp*sig
	dflux=flux*np.sqrt( (damp/amp)**2 + (dsig/sig)**2 )
	EW=flux/cont
	dEW=EW*np.sqrt( (damp/amp)**2 + (dsig/sig)**2 + (dinter/cont)**2 + (dWL*slop/cont)**2 + (WL*dslop/cont)**2 )

	print(pref,'\t{0} \t {4} \t {1} \t {5} \t {2} \t {6} \t {3} \t {7} \\\\'\

		.format('%.2f'%WL, '%.2f'%FWkms, '%.2f'%EW, '%.3f'%flux, '%.2f'%dWL, '%.2f'%dFWkms, '%.2f'%dEW, '%.2f'%dflux))

	



#*************#

#  PLOTTING   #

#*************#



# to have the single components for plot
comps = result.eval_components()



# 'auto scale' the plot depending on the flux min and max 
plmin=min(y)
plmax=max(y)

plt.rc('text', usetex=True)
plt.rc('font', family='serif', size=17)

fig=plt.figure(figsize=(8, 6))
gs1=gridspec.GridSpec(20,1)
ax1 = fig.add_subplot(gs1[:17,:])
ax2 = fig.add_subplot(gs1[17:,:])
#ax1.set_ylim(4.5,15.)
ax1.set_ylim(plmin-3,plmax+15)
ax2.set_ylim(-10.0,10.0)
ax1.tick_params(labelbottom=False)
ax1.set_ylabel('Flux [1E-16 erg/$\mathrm{cm^2}$/s/\AA]', size=20)
ax2.set_xlabel('Restframe Wavelength [\AA]', size=20)
ax1.errorbar(x,y,ey, fmt='k',lw=1,zorder=100, capsize=2)
#ax1.plot(x,y,'k',lw=1,zorder=100)
ax1.plot(x,result.best_fit,'m',lw=2)

# single components

for pref in prefixes:
	ax1.plot(x, comps[pref]+comps['poly'],linestyle='--')

	
#best fit	
ax2.plot(x,result.best_fit-y,'k', linestyle=':')


# this if you want to how the 3-sigma 'shade' around the best model fit
dely = result.eval_uncertainty(sigma=3)
ax1.fill_between(x, result.best_fit-dely, result.best_fit+dely, color="#ABABAB")

title = infile.name
plt.savefig("fit_"+title+"_Hbeta_OIII.pdf", dpi=800)
plt.show()



